
/******************************************************************
 **
 **			   PROPRIETARY NOTICE
 **
 **     Copyright (c) 1995-2001
 **         By NMS Communications, Inc.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of NMS Communications, Inc.
 **
 **  98/02/25 - PHY -	Added code to handle lightweight threads for
 **			Solaris port
 **
 *****************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <stropts.h>
#include <poll.h>
#include <errno.h>
#include <stdlib.h>

/*#pragma pack(1) */
#include <dpruser.h>
#include <txcpi.h>
#include <tx_core.h>

/* #pragma pack() */

#ifdef UNIX

typedef struct txcore_glob
{
    short           (*hdlr[FOPEN_MAX]) (TX_HANDLE, short);
    struct pollfd   fds[FOPEN_MAX];
    TX_HANDLE       handles[FOPEN_MAX];
    short           numfds;
}
txcore_glob_t;

#define PTHREAD_GLOB_DECL txcore_glob_t *gp=pthread_getspecific(txcore_key); \
                        if (gp == NULL) return TX_CORE_ERROR;

#define GLOB(x)		(gp->x)

#else   /* UNIX */

#define	PTHREAD_GLOB_DECL
#define GLOB(x)		x

extern int      errno;
extern CPI_ERR_TYPE cpi_errno;

/********************************
 * NOTE - to remain thread-safe, any globals or statics added here *must*
 *		be added to the txcore_glob_t definition above.
 ********************************/
static short    (*hdlr[FOPEN_MAX]) (TX_HANDLE fd, short events);
static struct pollfd fds[FOPEN_MAX];
static TX_HANDLE handles[FOPEN_MAX];
static short    numfds = 0;
#endif  /* UNIX */

/*
 * For sun - std* is thread-safe as long as the variables
 * remain read-only.
 */
CPI_WAIT_TYPE   stdInput = 0;
CPI_WAIT_TYPE   stdOutput = 1;
CPI_WAIT_TYPE   stdError = 2;

#ifdef UNIX
static pthread_once_t once_control = PTHREAD_ONCE_INIT;
static pthread_key_t txcore_key;

void txcore_thread_cleanup(void *argp)
{
    txcore_glob_t  *globp = (txcore_glob_t *) argp;

    if (argp != NULL)
        free((caddr_t) argp);
}

/******************************
 * This is called only once per process. (pthread_once(3T))
 ******************************/
void txcore_init_key(void)
{
    /*
     * If there's an error,it will be caught in init
     */
    pthread_key_create(&txcore_key, txcore_thread_cleanup);
}

int txcore_thread_init()
{
    int             thread_err;
    txcore_glob_t  *gp;

    pthread_once(&once_control, txcore_init_key);
    gp = (txcore_glob_t *) pthread_getspecific(txcore_key);
    if (gp != NULL)
        return TX_CORE_SUCCESS;     /* here once before in same thread */

    gp = (txcore_glob_t *) malloc(sizeof(*gp));
    if (gp == NULL)
        return TX_CORE_ENOMEM;

    bzero((caddr_t) gp, sizeof(*gp));
    thread_err = pthread_setspecific(txcore_key, (void *)gp);
    switch (thread_err)
    {
        case ENOMEM:
        case EAGAIN:
            return TX_CORE_ENOMEM;
        default:
            return TX_CORE_SUCCESS;
    }
}

#endif  /* UNIX */

short tx_core_kbhit()
{
    return (TRUE);
}

short tx_core_set_poll(CPI_WAIT_TYPE fd, short events)
{
    short           i;

    PTHREAD_GLOB_DECL;

    for (i = 0; i < GLOB(numfds); i++)
    {
        if (GLOB(fds)[i].fd == fd)
        {
            if (GLOB(hdlr)[i] == NULL)
                return (ENODEV);
            else
            {
                GLOB(fds)[i].events = events;
                return (TX_CORE_SUCCESS);
            }
        }
    }
    return (TX_CORE_ERROR);
}

short tx_core_process_events(short event, short revents)
{

    if (event & revents)
        return (TRUE);

    return (FALSE);
}

short
tx_core_install_handler(CPI_WAIT_TYPE fd,
                        TX_HANDLE handle,
                        short (*handler) (TX_HANDLE fd, short events))
{
    short           i;

    PTHREAD_GLOB_DECL;

    for (i = 0; i < FOPEN_MAX; i++)
    {
        if (GLOB(fds)[i].fd == -1)
        {
            GLOB(hdlr)[i] = handler;
            GLOB(fds)[i].fd = fd;
            GLOB(handles)[i] = handle;
            ++GLOB(numfds);
            return (TX_CORE_SUCCESS);
        }
        if (GLOB(fds)[i].fd == fd)
            return (TX_CORE_ERROR);
    }
}

short tx_core_init()
{
    short           i;

#ifdef UNIX                         /* Thread-safe initialization */
    int             err;
    txcore_glob_t  *gp;

    err = txcore_thread_init();
    if (err != TX_CORE_SUCCESS)
        return err;
    gp = (txcore_glob_t *) pthread_getspecific(txcore_key);
#endif  /* UNIX */

    for (i = 0; i < FOPEN_MAX; i++)
    {
        GLOB(hdlr)[i] = NULL;
        GLOB(fds)[i].fd = -1;
        GLOB(fds)[i].events = 0;
        GLOB(fds)[i].revents = 0;
        GLOB(handles)[i] = CPI_INVALID_HANDLE;
    }
    GLOB(numfds) = 0;

    setbuf(stdout, NULL);
    setbuf(stdin, NULL);

    return (TX_CORE_SUCCESS);
}

S32 tx_core_poll_loop(S32 wait_time)
{
    short           i;
    int             ret;
    int             timeout = (int)wait_time;

    PTHREAD_GLOB_DECL;

    for (;;)
    {
        if ((ret = poll(GLOB(fds), GLOB(numfds), timeout)) < 0)
        {
            CPI_SETERR(errno);
            return (TX_CORE_ERROR);
        }

        if (ret == 0)
        {
            CPI_SETERR(ETIME);
            return (TX_CORE_TIMEOUT);
        }

        for (i = 0; i < GLOB(numfds); i++)
        {
            if (GLOB(fds)[i].revents)
            {
                if (GLOB(hdlr)[i] != NULL)
                {
                    int             ret;

                    if ((ret = (*GLOB(hdlr)[i])
                         (GLOB(handles)[i],
                          GLOB(fds)[i].revents)) != TX_CORE_SUCCESS)
                    {
                        return (ret);
                    }
                }
            }
        }                           /* for i */
    }                               /* for ever */
}

char           *tx_core_gets(char *input, short len)
{
    char *fgets_result;

    fgets_result = fgets(input, (int)len, stdin);
    while ( (strlen(input) > 0) && (input[strlen(input) - 1] < ' ') )
        input[strlen(input) - 1] = '\0';    /* remove any trailing non-printable characters */
    return(fgets_result);
}

int stricmp(char *s1, char *s2)
{
    for (; tolower(*s1) == tolower(*s2); s1++, s2++)
    {
        if (*s1 == '\0')
        {
            return (0);
        }
    }

    return (tolower(*s1) - tolower(*s2));
}
